SystemOrganization addCategory: #'Athens-Core'!
SystemOrganization addCategory: #'Athens-Core-Matrices'!
SystemOrganization addCategory: #'Athens-Core-Paints'!
SystemOrganization addCategory: #'Athens-Core-Paths'!

!Color methodsFor: '*Athens-Core' stamp: 'IgorStasenko 4/13/2011 17:47'!
asAthensPaintOn: anAthensCanvas
	^ anAthensCanvas surface createSolidColorPaint: self! !

!Rectangle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 3/22/2011 22:04'!
asAthensPathOn: anAthensCanvas 
	^ anAthensCanvas createRectanglePath: self! !

!Rectangle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 3/27/2011 18:43'!
asPolygonTransformedBy: aTransform
	"answer a clock-wise oriented polygon, representing receiver,
	transformed by transform"
	
	^ {
		aTransform transform: origin.
		aTransform transformX: corner x Y: origin y.
		aTransform transform: corner.
		aTransform transformX: origin x Y: corner y.
	}! !

!Rectangle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 3/23/2011 03:00'!
paintFillsUsing: aPaint on: anAthensCanvas 
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	^ aPaint fillRectangle: self on: anAthensCanvas! !

!ImageFillStyle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 4/22/2011 11:14'!
asAthensPaintOn: anAthensCanvas
	
	^ (anAthensCanvas surface
		createImagePaint: form)
		origin: origin;
		direction: (form width@0)! !

!SolidFillStyle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 4/21/2011 15:01'!
asAthensPaintOn: anAthensCanvas 

	^ anAthensCanvas surface createSolidColorPaint: color! !

!Form methodsFor: '*Athens-Core' stamp: 'IgorStasenko 3/8/2012 13:45'!
asAthensPaintOn: canvas 

	^ canvas surface createFormPaint: self! !

!GradientFillStyle methodsFor: '*Athens-Core' stamp: 'IgorStasenko 4/21/2011 15:25'!
asAthensPaintOn: anAthensCanvas

	^ radial ifTrue: [
		anAthensCanvas surface
			createCircularGradient: colorRamp
			center: origin
			radius: direction y ]
	ifFalse: [
		anAthensCanvas surface
			createLinearGradient: colorRamp
			origin: origin
			corner: origin + direction ]
! !

Object subclass: #AthensAbstractPaint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!AthensAbstractPaint commentStamp: 'FernandoOlivero 3/9/2012 14:03' prior: 0!
I am an abstract paint, which should be converted before it can be used by Athens.
I am backend agnostic, as opposed to concrete paint(s).
Also, any other object can play the role of paint, as long as it implements the conversion method, #asAthensPaintOn:

See other implementors of #asAthensPaintOn:, such as Color and Form.!

!AthensAbstractPaint methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 3/9/2012 13:30'!
asAthensPaintOn: anAthensCanvas	
	^ self! !

AthensAbstractPaint subclass: #FileImagePaint
	instanceVariableNames: 'fileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!FileImagePaint classSide methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 1/12/2012 12:47'!
fileNamed: aFileName
	| image|
	image := self new.
	image initializeOnFileNamed: aFileName.
	^ image! !

!FileImagePaint methodsFor: 'converting' stamp: 'FernandoOlivero 1/25/2012 00:42'!
asAthensPaintOn: anAthensCanvas
	 ^anAthensCanvas surface createImagePaint: fileName! !

!FileImagePaint methodsFor: 'initialize-release' stamp: 'FernandoOlivero 1/12/2012 12:47'!
initializeOnFileNamed: aFileName
	fileName := aFileName.! !

AthensAbstractPaint subclass: #PatternPaint
	instanceVariableNames: 'stops colorRamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

PatternPaint subclass: #LinearPaint
	instanceVariableNames: 'start stop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!LinearPaint classSide methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 1/13/2012 23:10'!
from: aStart  to: aStop
	| p |
	p := self new.
	p initializeFrom: aStart  to: aStop.
	^ p! !

!LinearPaint methodsFor: 'converting' stamp: 'FernandoOlivero 2/24/2012 19:52'!
asAthensPaintOn: aCanvas
	^ aCanvas surface
			createLinearGradient: self colorRamp
				origin: self start 
				corner: self stop! !

!LinearPaint methodsFor: 'initialize-release' stamp: 'FernandoOlivero 1/13/2012 23:10'!
initializeFrom: aStart  to: aStop
	start := aStart.
	stop := aStop.! !

!LinearPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 23:10'!
start

	^ start! !

!LinearPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 23:10'!
stop

	^ stop! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 22:45'!
addStopAt: anOffset colored: aColor
	stops add: (anOffset -> aColor).
	colorRamp := stops asArray.! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 22:45'!
addStopAt: anOffset rgb: anArray
	self addStopAt: anOffset colored: anArray.! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 22:45'!
addStopAt: anOffset rgba: anArray
	self addStopAt: anOffset colored: anArray.! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 20:45'!
clear
	stops := OrderedCollection new.! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 21:11'!
colorRamp
	^ colorRamp! !

!PatternPaint methodsFor: 'initialize-release' stamp: 'FernandoOlivero 1/13/2012 20:33'!
initialize
	super initialize.
	stops := OrderedCollection new.! !

!PatternPaint methodsFor: 'testing' stamp: 'FernandoOlivero 1/13/2012 21:36'!
is: aSymbol
	^ aSymbol = #pattern or:[ super is: aSymbol]
	! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 20:32'!
stops

	^ stops! !

!PatternPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 21:11'!
stops: anObject

	stops := anObject.
	colorRamp := stops asArray.! !

PatternPaint subclass: #RadialPaint
	instanceVariableNames: 'inner outer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!RadialPaint classSide methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 1/13/2012 22:20'!
between: anOrigin extending: innerRadius to: outerPosition extending: outerRadius
	| paint |
	paint := self new.
	paint initializeFrom: anOrigin extending: innerRadius to: outerPosition extending: outerRadius.
	^ paint! !

!RadialPaint methodsFor: 'converting' stamp: 'FernandoOlivero 2/24/2012 19:52'!
asAthensPaintOn: aCanvas
	^ aCanvas surface
		createCircularGradient:  colorRamp
		between:  inner key 
		radius:  inner value 
		and:  outer key
		radius:  outer value! !

!RadialPaint methodsFor: 'initialize-release' stamp: 'FernandoOlivero 1/13/2012 21:25'!
initializeFrom: anOrigin extending: innerRadius to: outerPosition extending: outerRadius
	inner :=  anOrigin ->  innerRadius.
	outer := outerPosition ->  outerRadius.! !

!RadialPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 20:40'!
inner

	^ inner! !

!RadialPaint methodsFor: 'accessing' stamp: 'FernandoOlivero 1/13/2012 20:40'!
outer

	^ outer! !

Object subclass: #AthensCanvas
	instanceVariableNames: 'paint surface shape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensCanvas classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/21/2011 00:19'!
new
	"use #on: instead"
	self shouldNotImplement! !

!AthensCanvas classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/21/2011 00:17'!
on: aSurface
	^ self basicNew initializeWithSurface: aSurface! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:31'!
clearMask: aShape
	"set mask pixels to 0 for all pixels covered by shape, aShape"
	^ aShape paintFillsUsing: surface clearMaskPaint on: self
! !

!AthensCanvas methodsFor: 'clipping' stamp: 'IgorStasenko 4/21/2011 14:47'!
clipBy: aRectangle during: aBlock
	
	" Set a clipping rectangle during drawing operations performed in a block.
	Note that clipping rectangle are intetersected with currently active clipping
	rectangle "
	
	 ^surface clipBy: aRectangle during: aBlock! !

!AthensCanvas methodsFor: 'paths' stamp: 'IgorStasenko 3/7/2012 14:44'!
createPath: aPathCreatingBlock
	
	^ surface createPath: aPathCreatingBlock! !

!AthensCanvas methodsFor: 'paths' stamp: 'IgorStasenko 3/22/2011 22:04'!
createRectanglePath: aRectangle 
	^ surface createRectanglePath: aRectangle 
! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:06'!
disableMask
	^ surface disableMask! !

!AthensCanvas methodsFor: 'drawing' stamp: 'FernandoOlivero 3/13/2012 16:21'!
draw
	"Fill the shape (anObject) using currently selected paint
	an object should implement double-dispatch to currently selected paint"
	
	^ shape paintFillsUsing: paint on: self

! !

!AthensCanvas methodsFor: 'drawing' stamp: 'IgorStasenko 3/20/2011 23:51'!
draw: anObject

	^ anObject drawOnAthensCanvas: self! !

!AthensCanvas methodsFor: 'drawing' stamp: 'FernandoOlivero 3/13/2012 16:21'!
drawShape: anObject
	self setShape: anObject.
	self draw.
	! !

!AthensCanvas methodsFor: 'morphic' stamp: 'IgorStasenko 11/21/2011 16:24'!
drawString: aString in: aRectangle font: aFont color: aColor 
	
	| renderer |
	renderer := aFont glyphRendererOn:self surface.
	
	renderer 
		setColor: aColor;
		setPosition: aRectangle origin;
		renderGlyphsIn: aString from: 1 to: aString size
! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:06'!
enableMask
	^ surface enableMask! !

!AthensCanvas methodsFor: 'morphic' stamp: 'IgorStasenko 4/21/2011 14:27'!
fullDrawMorph: aMorph

"Hook method for potential other canvases. In the core, this method looks supefluous but PostscriptCanvases and other canvases can specialized it for Morph rendering. Therefore it should not be merged with fullDraw:."

	aMorph fullDrawOnAthensCanvas: self! !

!AthensCanvas methodsFor: 'initialization' stamp: 'IgorStasenko 3/22/2011 11:13'!
initializeWithSurface: anAthensSurface	
	surface := anAthensSurface! !

!AthensCanvas methodsFor: 'clipping' stamp: 'IgorStasenko 4/21/2011 14:40'!
isVisible: aRectangle 
	"check if rectangle are not fully clipped by current clipping coordinates"
	
	^ true! !

!AthensCanvas methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 12:12'!
paint
	^ paint  ! !

!AthensCanvas methodsFor: 'transformation matrices' stamp: 'IgorStasenko 3/26/2011 00:38'!
paintTransform
	^ surface paintTransform
! !

!AthensCanvas methodsFor: 'transformation matrices' stamp: 'IgorStasenko 3/26/2011 00:37'!
pathTransform
	^ surface pathTransform
! !

!AthensCanvas methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:35'!
setMask: aShape
	"set mask pixels to 1 for all pixels covered by shape, aShape"

	^ aShape paintFillsUsing: surface setMaskPaint on: self
! !

!AthensCanvas methodsFor: 'paint' stamp: 'FernandoOlivero 1/25/2012 00:46'!
setPaint: aPaint
	paint = aPaint ifTrue:[ ^ paint ].
	paint finalize.
	paint := aPaint asAthensPaintOn: self.
	^ paint.! !

!AthensCanvas methodsFor: 'accessing' stamp: 'FernandoOlivero 3/13/2012 16:20'!
setShape: anObject
	shape := anObject.! !

!AthensCanvas methodsFor: 'paint' stamp: 'FernandoOlivero 1/13/2012 18:58'!
setStrokePaint: aPaint
	paint finalize.
	paint := (aPaint asAthensPaintOn: self) asStrokePaint.
	^ paint! !

!AthensCanvas methodsFor: 'accessing' stamp: 'IgorStasenko 3/22/2011 22:39'!
surface
	^ surface! !

Object subclass: #AthensPaint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paints'!

!AthensPaint commentStamp: 'FernandoOlivero 3/9/2012 13:57' prior: 0!
I am a base class, which defines a paint.

My subclasses represent a concreate paint by implementing my protocol,  that actually draws a specific shapes with me. My subclasses usually backend-specific, but as long as you implement my protocol, any object can be treated as paint:

 - fillPath: aPath on: aCanvas
 - fillRectangle: aRectangle on: aCanvas
 - asStrokePaint





!

!AthensPaint methodsFor: 'converting' stamp: 'FernandoOlivero 3/9/2012 13:31'!
asStrokePaint
	self subclassResponsibility! !

!AthensPaint methodsFor: 'drawing' stamp: 'IgorStasenko 5/1/2011 23:20'!
fillPath: athensBalloonPath on: anAthensCanvas 

	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	self subclassResponsibility! !

!AthensPaint methodsFor: 'drawing' stamp: 'IgorStasenko 3/23/2011 03:02'!
fillRectangle: aRect on: anAthensCanvas
	"This is a terminal method in rendering dispatch scheme
	canvas->shape->paint. See AthensCanvas>>fillShape: "
	
	self subclassResponsibility! !

AthensPaint subclass: #AthensStrokePaint
	instanceVariableNames: 'fillPaint width capStyle joinStyle innerJoinStyle miterLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
capStyle

	^ capStyle! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
capStyle: anObject

	capStyle := anObject! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
fillPaint

	^ fillPaint! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
fillPaint: anObject

	fillPaint := anObject! !

!AthensStrokePaint methodsFor: 'initialize-release' stamp: 'FernandoOlivero 1/13/2012 00:35'!
initialize
	super initialize.
	width := 1.
	capStyle := #butt.
	joinStyle := #bevel.
	innerJoinStyle := #bevel.
	miterLimit := 4.0.
	! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
innerJoinStyle

	^ innerJoinStyle! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
innerJoinStyle: anObject

	innerJoinStyle := anObject! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
joinStyle

	^ joinStyle! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
joinStyle: anObject

	joinStyle := anObject! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
miterLimit

	^ miterLimit! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
miterLimit: anObject

	miterLimit := anObject! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 9/19/2011 15:40'!
width

	^ width! !

!AthensStrokePaint methodsFor: 'accessing' stamp: 'IgorStasenko 4/29/2011 15:28'!
width: anObject

	width := anObject! !

Object subclass: #AthensPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensPath commentStamp: 'IgorStasenko 3/21/2011 03:48' prior: 0!
Internally, all shapes in Athens are converted into paths before drawing.
Path data organization is up to surface where it built,
therefore paths are not interchangeable between different surfaces.!

!AthensPath methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/22/2011 22:47'!
numSegments
	"answer a number of segments path contains"
	
	self subclassResponsibility! !

Object subclass: #AthensPathBuilder
	instanceVariableNames: ''
	classVariableNames: 'ZeroPoint'
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPathBuilder commentStamp: 'IgorStasenko 3/7/2012 14:32' prior: 0!
i am abstract path builder, to demonstrate an interface protocol which is available for building paths,
by using  canvas #createPath: method.!

!AthensPathBuilder classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/7/2012 15:26'!
initialize
	ZeroPoint := 0@0! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:26'!
absolute
	"next commands will use absolute coordinates for all segment points"
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:28'!
ccwArcTo: endPt angle: rot
	" Add a counter-clockwise arc segment, starting from current path endpoint and
	ending at andPt. Angle should be specified in radians "
	

	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:30'!
close
	"close the current path controur"
	self subclassResponsibility ! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:30'!
curveVia: cp1 and: cp2 to: aPoint
	"Add a cubic bezier curve,
	starting from current path endpoint, using control points cp1, cp2
	and ending at aPoint "
	
	self subclassResponsibility 	! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:29'!
curveVia: cp1 to: aPoint

	"Add a quadric bezier curve,
	starting from current path endpoint, using control point cp1, 
	and ending at aPoint "
	
	self subclassResponsibility ! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:28'!
cwArcTo: endPt angle: rot
	" Add a clockwise arc segment, starting from current path endpoint and
	ending at andPt. Angle should be specified in radians "
	

	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:27'!
lineTo: aPoint
	"add a line segment, starting from current path endpoint to aPoint"
	
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:27'!
moveTo: aPoint
	" move command always starts a new contour "
	self subclassResponsibility! !

!AthensPathBuilder methodsFor: 'path commands' stamp: 'IgorStasenko 3/7/2012 14:27'!
relative

	"next commands will use relative coordinates for all segment points"
	self subclassResponsibility! !

Object subclass: #AthensPathConverter
	instanceVariableNames: 'sourcePath dest endPoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPathConverter commentStamp: 'IgorStasenko 5/1/2011 23:14' prior: 0!
i am a base class for path conversion. 
i take a path as input and producing a path commands as output.

My default implementation just passing all path commands without changes to destination without any conversion

Example of use:

 
converter := AthensXYZConverter dest: (AthensPolygonPath new).
convertedPath := converter convert: sourcePath.

You can chain multiple converters:

basicConverter := AthensZYXConv dest: AthensPolygonPath new. 
compoundConverter := AthensABCConverter dest: basicConverter.

compoundConverter convert: somePath

in this case, an initial conversion is performed by instance of AthensABCConverter, and then conversion results are passed down to AthensZYXConv,
and then finally to instance of AthensPolygonPath.
!

AthensPathConverter subclass: #AthensBezierConverter
	instanceVariableNames: 'distanceTolerance angleTolerance'
	classVariableNames: 'CollinearityEps CurveAngleTolerance DistanceEps'
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensBezierConverter commentStamp: 'IgorStasenko 5/1/2011 21:50' prior: 0!
I converting all bezier segments to a sequence of line segments, by approximating the curve along its path.

This classs is used for generating strokes.!

!AthensBezierConverter classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/29/2011 18:10'!
initialize

	CollinearityEps := 1e-30.
	DistanceEps := 1e-30.
	CurveAngleTolerance := 0.01.! !

!AthensBezierConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 5/1/2011 21:45'!
angleBetween: p1 and: p2 ifDegenerate: aBlock

" Calculate an angle (in radians) between two vectors. 
Evaluate a block, in case if calculation not possible because one of the vectors has zero length "

	| x1 y1 x2 y2 dot2 n2 |
	x1 := p1 x.
	y1 := p1 y.
	x2 := p2 x.
	y2 := p2 y.
	
	dot2 := x1 * x2 + (y1 * y2).
	dot2 := dot2 * dot2.
	
	n2 := (x1*x1 + (y1*y1)) * (x2*x2 + (y2*y2)).
	
	n2 = 0 ifTrue: [ ^ aBlock value ].
	
	^ (dot2 / n2) arcCos! !

!AthensBezierConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 5/1/2011 22:03'!
curveVia: pt1 to: pt2


	self	 recursiveBezier2_x1: endPoint x y1: endPoint y
		x2: pt1 x y2: pt1 y
		x3: pt2 x y3: pt2 y! !

!AthensBezierConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/29/2011 18:11'!
initialize

	distanceTolerance := 0.5.
	angleTolerance := 0.1 . ! !

!AthensBezierConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/29/2011 18:09'!
isFlatBezier2_x1: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3

	| dx dy d da angle |
	
	dx := x3-x1.
	dy := y3-y1.
	
 	d := (((x2 - x3) * dy) - ((y2 - y3) * dx)) abs.

	d > CollinearityEps ifTrue: [
		
		"regular case"

		d*d <= (distanceTolerance * ( dx*dx + (dy*dy))) ifTrue: [
			
			angleTolerance < CurveAngleTolerance ifTrue: [ ^ true ].
			
			angle := self angleBetween: x2-x1 @ (y2-y1) and: x3-x2 @ (y3-y2)
				ifDegenerate: [ 0.0 ].
			
			"parallel. no need to proceed further"
			angle <= angleTolerance ifTrue: [ ^ true ]					
		]
	]
	ifFalse: [ 
		"collinear"
		da := dx*dx + (dy*dy).
		
		da = 0 
			ifTrue: [ d := (x1-x2) squared + (y1-y2) squared ]
			ifFalse: [
				d = ((x2 - x1)*dx + ((y2 - y1)*dy)) / da.

				(d > 0.0 and: [ d < 1.0 ] ) ifTrue: [ 
					"Simple collinear case, 1---2---3" 
					^ true
     				].
				d <= 0.0 
					ifTrue: [ d := (x1-x2) squared + (y1-y2) squared ]
					ifFalse: [
						d >= 1.0 
							ifTrue: [ d:= (x2-x3) squared + (y2-y3) squared ]
							ifFalse: [ d:= (x2 - x1 - (d*dx)) squared + (y2 - y1 - (d*dy)) squared ]
					].
			].

			d < self distanceToleranceSquared ifTrue: [ ^ true ]		
	].

	^ false! !

!AthensBezierConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 5/1/2011 21:47'!
recursiveBezier2_x1: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3

	"recursively subdive bezier curve as long as #isFlatBezier2.. answers false "

	(self isFlatBezier2_x1: x1 y1: y1 x2: x2 y2: y2 x3: x3 y3: y3) ifTrue: [
		
		dest 
			lineTo: x2 @ y2;
			lineTo: x3 @ y3
	] ifFalse: [
		| x12 y12 x23 y23 x123 y123 |	
	"calculate midpoints of line segments "
		x12 := (x1 + x2) * 0.5.
		y12 := (y1 + y2) * 0.5 .

		x23 := (x2 + x3) * 0.5 .
		y23 := (y2 + y3) * 0.5 .

		x123 := (x12 + x23) * 0.5.
		y123 := (y12 + y23) * 0.5.
		
		self recursiveBezier2_x1: x1 y1: y1 
			x2: x12 
			y2: y12 
			x3: x123 
			y3: y123.
			
		self recursiveBezier2_x1: x123 
			y1: y123
			x2: x23
			y2: y23 
			x3: x3 
			y3: y3.
	]
! !

!AthensPathConverter classSide methodsFor: 'instance creation' stamp: 'IgorStasenko 5/1/2011 22:18'!
dest: aDest

	^ self new dest: aDest! !

!AthensPathConverter classSide methodsFor: 'instance creation' stamp: 'IgorStasenko 5/1/2011 22:20'!
source: aSource dest: aDest
	" answer a converted path from source to dest, using my instance as a converter"
	
	^ self new source: aSource dest: aDest! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 5/2/2011 05:18'!
close: aPoint 
	^ dest close: aPoint! !

!AthensPathConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 5/1/2011 22:16'!
convert: aSourcePath
	
	" iterate over segments and do conversion "
	
	| segment |
	
	segment := aSourcePath.
	
	[ segment notNil ] whileTrue: [
		segment convertWith: self.
		endPoint := segment endPoint.
		segment := segment next.
	].

	^ dest finish! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 4/29/2011 18:31'!
curveVia: pt1 to: pt2

	^ dest curveVia: pt1 to: pt2
! !

!AthensPathConverter methodsFor: 'accessing' stamp: 'IgorStasenko 5/1/2011 22:18'!
dest: anObject
	dest := anObject! !

!AthensPathConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 5/1/2011 22:13'!
finish
	^ dest finish! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 4/29/2011 18:36'!
lineTo: aPoint 
	^ dest lineTo: aPoint! !

!AthensPathConverter methodsFor: 'converting path commands' stamp: 'IgorStasenko 4/29/2011 18:28'!
moveTo: aPoint 
	^ dest moveTo: aPoint! !

!AthensPathConverter methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 4/29/2011 18:18'!
source: aSource dest: aDest

	sourcePath := aSource.
	dest := aDest! !

Object subclass: #AthensPathSegment
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

AthensPathSegment subclass: #AthensArcSegment
	instanceVariableNames: 'endPoint angle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensArcSegment commentStamp: 'IgorStasenko 3/7/2012 13:41' prior: 0!
i am abstract class , representing an arc segment!

!AthensArcSegment methodsFor: 'accessors' stamp: 'IgorStasenko 3/7/2012 13:44'!
angle
	^ angle! !

!AthensArcSegment methodsFor: 'accessors' stamp: 'IgorStasenko 3/7/2012 13:31'!
endPoint
	^ endPoint! !

!AthensArcSegment methodsFor: 'accessors' stamp: 'IgorStasenko 3/7/2012 13:30'!
endPoint: pt angle: a

	endPoint := pt.
	angle := a! !

AthensArcSegment subclass: #AthensCCWArcSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCCWArcSegment methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/7/2012 13:44'!
visitWith: anObject

	^ anObject visitCcwArcSegment: self! !

AthensArcSegment subclass: #AthensCWArcSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCWArcSegment methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/7/2012 13:44'!
visitWith: anObject

	^ anObject visitCwArcSegment: self! !

AthensPathSegment subclass: #AthensCubicSegment
	instanceVariableNames: 'via1 via2 to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCubicSegment commentStamp: 'IgorStasenko 3/7/2012 13:32' prior: 0!
i represent a cubic Bezier path segment!

!AthensCubicSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:16'!
convertWith: conv

	^ conv curveVia: via1 and: via2 to: to! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 11:39'!
endPoint
	^ to! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:17'!
from: origin
	via: controlPoint
	and: secondControlPoint
	to: destination
	
	
	via1:= controlPoint.
	via2 := secondControlPoint.
	to := destination.! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:05'!
to

	^ to! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:09'!
via1
	^ via1! !

!AthensCubicSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 2/14/2012 08:09'!
via2
	^ via2! !

!AthensCubicSegment methodsFor: 'visitor' stamp: 'IgorStasenko 3/26/2011 19:37'!
visitWith: anObject

	^ anObject visitCubicSegment: self! !

AthensPathSegment subclass: #AthensLineSegment
	instanceVariableNames: 'point'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

AthensLineSegment subclass: #AthensCloseSegment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensCloseSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:14'!
convertWith: conv

	^ conv close: point! !

!AthensCloseSegment methodsFor: 'visitor' stamp: 'IgorStasenko 3/26/2011 19:38'!
visitWith: anObject

	^ anObject visitCloseSegment: self! !

!AthensLineSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:14'!
convertWith: conv

	^ conv lineTo: point! !

!AthensLineSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:21'!
endPoint
	^ point! !

!AthensLineSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:16'!
point

	^ point! !

!AthensLineSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:16'!
point: anObject

	point := anObject! !

!AthensLineSegment methodsFor: 'visitor' stamp: 'IgorStasenko 3/26/2011 19:38'!
visitWith: anObject

	^ anObject visitLineSegment: self! !

AthensLineSegment subclass: #AthensMoveSegment
	instanceVariableNames: 'closed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensMoveSegment commentStamp: 'IgorStasenko 5/2/2011 05:35' prior: 0!
Move segment always indicates starting of a new contour.
If contour ends with close command, then my insntance will have 'closed' flag set to true.

See AthensPathBuilder , which is responsible for building paths.!

!AthensMoveSegment methodsFor: 'accessing' stamp: 'IgorStasenko 5/2/2011 05:37'!
closed: aBoolean
	closed := aBoolean! !

!AthensMoveSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:14'!
convertWith: conv

	^ conv moveTo: point! !

!AthensMoveSegment methodsFor: 'testing' stamp: 'IgorStasenko 5/2/2011 05:37'!
isClosed
	^ closed == true! !

!AthensMoveSegment methodsFor: 'testing' stamp: 'IgorStasenko 3/27/2011 17:48'!
isMove
	^ true! !

!AthensMoveSegment methodsFor: 'accessing' stamp: 'FernandoOlivero 1/15/2012 14:59'!
reopen: aBoolean
	closed := aBoolean! !

!AthensMoveSegment methodsFor: 'visitor' stamp: 'IgorStasenko 3/26/2011 19:38'!
visitWith: anObject

	^ anObject visitMoveSegment: self! !

!AthensPathSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:13'!
convertWith: anObject
	self subclassResponsibility! !

!AthensPathSegment methodsFor: 'testing' stamp: 'IgorStasenko 3/26/2011 16:44'!
isMove
	^ false! !

!AthensPathSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:14'!
next

	^ next! !

!AthensPathSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:14'!
next: anObject

	next := anObject! !

!AthensPathSegment methodsFor: 'visitor' stamp: 'IgorStasenko 3/26/2011 19:36'!
visitWith: anObject

	self subclassResponsibility! !

AthensPathSegment subclass: #AthensQuadSegment
	instanceVariableNames: 'via to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensQuadSegment commentStamp: 'IgorStasenko 3/7/2012 13:32' prior: 0!
i represent a quadric Bezier path segment.

Any quadratic spline can be expressed as a cubic (where the cubic term is zero). The end points of the cubic will be the same as the quadratic's.

    CP0 = QP0
    CP3 = QP2 

The two control points for the cubic are:

    CP1 = QP0 + 2/3 *(QP1-QP0)
    CP2 = QP2 + 2/3 *(QP1-QP2) !

!AthensQuadSegment methodsFor: 'visitor' stamp: 'IgorStasenko 4/29/2011 18:15'!
convertWith: conv

	^ conv curveVia: via to: to! !

!AthensQuadSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:52'!
endPoint
	^ to! !

!AthensQuadSegment methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/26/2011 16:52'!
from: pt0 via: pt1 to: p

	via := pt1.
	to := p.! !

!AthensQuadSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:52'!
to

	^ to! !

!AthensQuadSegment methodsFor: 'accessing' stamp: 'IgorStasenko 3/26/2011 16:52'!
via

	^ via! !

!AthensQuadSegment methodsFor: 'visitor' stamp: 'IgorStasenko 3/26/2011 19:38'!
visitWith: anObject

	^ anObject visitQuadSegment: self! !

Object subclass: #AthensPolygon
	instanceVariableNames: 'contours currentContour'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Paths'!

!AthensPolygon commentStamp: 'IgorStasenko 12/20/2011 18:38' prior: 0!
i am special kind of shape which using only straight line segments, unlike from more generic paths.!

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 5/1/2011 22:11'!
finish
	"Finally convert contours to arrays"
	
	contours := contours collect: [:ea | ea asArray ]! !

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 4/29/2011 18:32'!
lineTo: aPoint 
	currentContour add: aPoint! !

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 4/29/2011 18:29'!
moveTo: aPoint 
	" create a new contour "
	
	self newContour.
	currentContour add: aPoint.! !

!AthensPolygon methodsFor: 'conversion' stamp: 'IgorStasenko 4/29/2011 18:30'!
newContour
	contours ifNil: [ contours := OrderedCollection new ].
	
	currentContour := contours add: OrderedCollection new.! !

!AthensPolygon methodsFor: 'drawing' stamp: 'IgorStasenko 5/2/2011 04:48'!
paintFillsUsing: aPaint on: anAthensCanvas 
	
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	^ aPaint fillPolygon: self on: anAthensCanvas! !

Object subclass: #AthensShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensShape commentStamp: 'IgorStasenko 12/20/2011 18:36' prior: 0!
this is an abstract class to denote a common protocol, which must be supported by any shapes used for rendering by Athens!

!AthensShape methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 12/20/2011 18:36'!
paintFillsUsing: aPaint on: anAthensCanvas 
	"This method is a part of rendering dispatch  Canvas->receiver->paint"
	
	self subclassResponsibility! !

Object subclass: #AthensSurface
	instanceVariableNames: 'currentCanvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensSurface classSide methodsFor: 'instance creation' stamp: 'sig 2/29/2012 21:43'!
extent: aPoint
	"Create a new surface with given extent. 
	Note that some surfaces may not support this method, since they may represent an infinite surface."
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'converting' stamp: 'sig 2/29/2012 21:51'!
asForm
	"Answer a Form , which contains a bits, converted from surface "
	self subclassResponsibility! !

!AthensSurface methodsFor: 'public' stamp: 'IgorStasenko 12/20/2011 15:38'!
clear
	"clear the surface"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'clipping' stamp: 'IgorStasenko 4/21/2011 14:47'!
clipBy: aRectangle during: aBlockClosure 
	self subclassResponsibility.! !

!AthensSurface methodsFor: 'paints' stamp: 'FernandoOlivero 1/13/2012 21:18'!
createCircularGradient: colorRamp center: aCenter radius: aRadius
 
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 3/8/2012 13:46'!
createFormPaint: aForm 
	self subclassResponsibility.! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 4/13/2011 17:45'!
createLinearGradient: colorRamp origin: pt1 corner: pt2
 
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paths' stamp: 'IgorStasenko 3/26/2011 19:32'!
createPath: aPathBuilder
	"Create a path from provided path builder instance"
	self subclassResponsibility! !

!AthensSurface methodsFor: 'paths' stamp: 'FernandoOlivero 1/13/2012 03:08'!
createRectanglePath: aRectangle 
	^ aRectangle! !

!AthensSurface methodsFor: 'paints' stamp: 'IgorStasenko 12/20/2011 15:25'!
createSolidColorPaint: aColor 
	 "Answer an instance of AthensPaint, valid for use with given surface"
	
	self subclassResponsibility! !

!AthensSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:15'!
disableMask
	self subclassResponsibility! !

!AthensSurface methodsFor: 'public' stamp: 'IgorStasenko 3/7/2012 15:06'!
drawDuring: aBlock

	"You may draw on receiver only when inside a block and only using provided canvas object.
	This ensures releasing system resources used after finishing drawing"


	currentCanvas ifNotNil: [ self attemptToRecurseDrawing ].
	
	[
		currentCanvas := self newCanvas.
		aBlock value: currentCanvas.
	] ensure: [
		self releaseCanvas.
		currentCanvas := nil.
	].! !

!AthensSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:15'!
enableMask
	self subclassResponsibility! !

!AthensSurface methodsFor: 'masking' stamp: 'IgorStasenko 3/27/2011 18:16'!
maskEnabled
	self subclassResponsibility! !

!AthensSurface methodsFor: 'private' stamp: 'IgorStasenko 3/21/2011 02:35'!
newCanvas
	"Answer a preinitialized instance of AthensCanvas.
	Private to receiver and its subclasses, override seldom"
	^ AthensCanvas on: self
! !

!AthensSurface methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/22/2011 22:40'!
releaseCanvas
	currentCanvas := nil! !

Object subclass: #AthensTransform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core'!

!AthensTransform commentStamp: 'IgorStasenko 3/7/2012 14:41' prior: 0!
My instances represent an interface for coordinate system transformation defined by 3x3 matrix of following kind:

| sx   shx   x   |
| shy sy     y   |
|  w0  w1  w2 |

sx and sy define scaling in the x and y directions, respectively;
shx and shy define shearing in the x and y directions, respectively;
tx and ty define translation in the x and y directions, respectively.

AthensSurface are responsible for providing specific implementation 
of me, which is most appropriate for backend its using.

My internal representation isnt available for manipulation, for example dont assume i'm a matrix. In short, do not copy my state or modify it directly, instead use methods such as #loadAffineTransformation:, and talk to my surface.

My subclasses should implement a common protocol for applying generic types of affine transformations
on coordinate system:
- translate
- rotate
- shear
- scale
- matrix multiply
- matrix load

Surface must support 4 kinds of transformations of coordinate system(s):
- path-to-surface (#pathTransform) to transform path coordinates to surface space
- image-to-surface (#imageTransform) to transform between user coordinates and surface pixels

- fill-paint-to-user (#fillTransform)
- stroke-paint-to-user (#strokeTransform)

Given a (fill or stroke) paint-to-user transformation Tp and user-to-surface transformation Tu, the paint color and alpha of a pixel to be drawn with surface coordinates (x, y) is defined by mapping its center point (x + 1/2, y + 1/2) through the inverse transformation (Tu * Tp)^-1 , resulting in a sample point in the paint coordinate space.

All transformations, except image-to-surface , ignoring w0 , w1 and w2 values and always assume them set to { 0 , 0 , 1 } respectively.

For accessing a particular kind of transformation, send message to canvas, i.e.:

canvas pathTransform translateBy: 10@10.

It is safe to store transformation in temporary variable, as long as canvas is valid.
!

AthensTransform subclass: #AthensAffineTransform
	instanceVariableNames: 'sx shx x shy sy y'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Matrices'!

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 10/21/2011 12:55'!
clearTranslation
	"reset a translation from matrix, leaving only scale and rotation"
	
	x := y := 0.0! !

!AthensAffineTransform methodsFor: 'initialize-release' stamp: 'IgorStasenko 3/26/2011 02:09'!
initialize
	self loadIdentity! !

!AthensAffineTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/28/2012 16:27'!
inverseTransform: aPoint 
	| px py y0 x0 |
	
	px := aPoint x.
	py := aPoint y.

"	x0 := px - x - (shx*y0) / sx.
	y0 := py - y - (shy*x0) / sy.
	
	y0 := py - y - (shy*(px - x - (shx*y0) / sx)) / sy.

	sy * y0  == py - y - (shy*(px - x - (shx*y0) / sx)).

	sx * sy * y0  == (sx*(py - y)) - (shy*(px - x - (shx*y0))).

	sx * sy * y0  == sx*py - (sx*y) - (shy*px) + (shy*x) + (shy*shx*y0).

	(sx * sy * y0) - (shy*shx*y0)  == sx*py - (sx*y) - (shy*px) + (shy*x) .
	
	y0* ((sx * sy) - (shy*shx))  == sx*py - (sx*y) - (shy*px) + (shy*x) .

"
	y0  := sx*py - (sx*y) - (shy*px) + (shy*x) / ((sx * sy) - (shy*shx)).
	x0 := px - x - (shx*y0) / sx.

	^ x0@y0! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 3/28/2012 15:10'!
inverted
	"answer an inverse transformation of receiver"
	self notYetImplemented ! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 14:02'!
loadAffineTransform: m

	x := m x.
	y := m y.
	sx := m sx.
	sy := m sy.
	shx := m shx.
	shy := m shy.! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 02:09'!
loadIdentity
	"initialize with identity transform"
	sx := sy := 1.0.
	shx := shy := x := y := 0.0.
	
	! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 14:01'!
restoreAfter: aBlock
	|previous|
	previous := self copy.
	aBlock ensure: [ 	self loadAffineTransform: previous ]! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:46'!
rotateByDegrees: angle


	^ self rotateByRadians: angle degreesToRadians! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 03:21'!
rotateByRadians: angle

	"multiply receiver by rotation matrix

| sx   shx   x   |        | cos -sin 0  |             | (sx*cos)+(shx*sin)   (-sx*sin+shx*cos)  x |
| shy sy     y   |   *   | sin   cos 0 |   ===> | (shy*cos)+(sy*sin)   (-shy*sin)+sy*cos) y |
|  0     0     1   |       | 0     0   1   |             |        0                                 0                1  |

"
	| cos sin newSx newSy |
	
	cos := angle cos.
	sin := angle sin.
	
	
	newSx := sx*cos + (shx*sin).
	newSy := sy*cos - (shy*sin).
	
	shx := shx*cos - (sx*sin).
	shy := shy*cos + (sy*sin).

	sx := newSx.
	sy := newSy.! !

!AthensAffineTransform methodsFor: 'accessing' stamp: ''!
scale
	^ sx@sy! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/21/2012 18:26'!
scaleBy: factor

	"multiply receiver by uniform scale matrix

| sx   shx   x   |       | (f x) 0 0  |             | sx*(f x)    shx*(f y)  x |
| shy sy     y   |   *   | 0 (f y) 0  |    ===> | shy*(f x)     sy*(f y)    y |
|  0     0     1  |        | 0 0     1  |                |  0      0              1  |

"	
	factor isPoint ifTrue: [
		sx := sx*factor x.
		shx := shx*factor y.
		sy := sy*factor y.
		shy := shy*factor x. 		
	] ifFalse: [
		sx := sx*factor.
		shx := shx*factor.
		sy := sy*factor.
		shy := shy*factor. ]
! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:52'!
scaleX: fx Y: fy

	"multiply receiver by scale matrix

| sx   shx   x   |        | fx 0 0  |              | sx*fx    shx*fx  x |
| shy sy     y   |   *   | 0 fy 0  |    ===> | shy*fy  sy*fy   y |
|  0     0     1   |       | 0 0  1  |              |  0          0        1  |

"
	sx := sx*fx.
	shx := shx*fx.
	sy := sy*fy.
	shy := shy*fy.
! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
shx
	^ shx! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
shy
	^ shy! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
sx
	^ sx! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
sy
	^ sy! !

!AthensAffineTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/26/2011 03:02'!
transform: aPoint 
	| px py |
	
	px := aPoint x.
	py := aPoint y.
	^ Point 
		x: (sx*px +(shx*py) + x)
		y: (shy*px + (sy*py) + y) ! !

!AthensAffineTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/27/2011 18:40'!
transformX: px Y: py
	" transform x and y coordinates by receiver. Answer a Point"
	^ Point 
		x: (sx*px +(shx*py) + x)
		y: (shy*px + (sy*py) + y) ! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 03:17'!
translateBy: aPoint

	^ self translateX: aPoint x Y: aPoint y! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/21/2012 18:23'!
translateX: px Y: py


" multiply receiver by translation matrix :

| sx   shx   x   |        | 1  0  px |             |	sx  shx  (sx*px + shx*py + x) |
| shy sy     y   |   *   | 0  1  py |   ===> | shy  sy  (shy*px + sy*py + y)  |
|  0     0     1   |       | 0  0   1  |             |  0     0    1  |

"
	x := (sx*px) + (shx*py) + x.
	y := (shy*px) + (sy*py) + y.
! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 10/24/2011 17:24'!
translateX: px y: py

" multiply receiver by translation matrix :

| sx   shx   x   |        | 1  0  px |             |	sx  shx  (sx*px + shx*py + x) |
| shy sy     y   |   *   | 0  1  py |   ===> | shy  sy  (shy*px + sy*py + y)  |
|  0     0     1   |       | 0  0   1  |             |  0     0    1  |

"
	x := (sx*px) + (shx*py) + x.
	y := (shy*px) + (sy*py) + y.
! !

!AthensAffineTransform methodsFor: 'accessing' stamp: ''!
translation
	^ x@y! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 3/28/2012 15:36'!
translation: aPoint
	x := aPoint x.
	y := aPoint y.! !

!AthensAffineTransform methodsFor: 'transformations' stamp: 'IgorStasenko 10/25/2011 13:20'!
transposed
	
	| s |
	s := shx.
	shx := shy.
	shy := s.! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
x
	^ x! !

!AthensAffineTransform methodsFor: 'accessing' stamp: 'IgorStasenko 10/20/2011 18:48'!
y
	^ y! !

AthensAffineTransform subclass: #AthensGenericTransform
	instanceVariableNames: 'w0 w1 w2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Core-Matrices'!

!AthensGenericTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 02:10'!
loadIdentity
	"initialize with identity transform"
	super loadIdentity.
	w0 := w1 := 0.0.
	w2 := 1.0! !

!AthensTransform methodsFor: 'transformations' stamp: ''!
invert
	"i should invert"
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 14:00'!
loadAffineTransform: anAthenTransform
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 12/20/2011 16:46'!
loadIdentity
	"load the identity matrix into receiver"
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/7/2012 13:52'!
restoreAfter: aBlock
	"i should save the current transform state, evaluate the block, and then restore the saved transform state"
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:56'!
rotateByDegrees: angle

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
rotateByRadians: angle

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
scaleBy: factor

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
scaleX: fx Y: fy

	self subclassResponsibility! !

!AthensTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/27/2011 18:42'!
transform: aPoint 
	self subclassResponsibility! !

!AthensTransform methodsFor: 'vector-transform' stamp: 'IgorStasenko 3/27/2011 18:42'!
transformX: px Y: py 
	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 3/26/2011 01:55'!
translateBy: aPoint

	self subclassResponsibility! !

!AthensTransform methodsFor: 'transformations' stamp: 'IgorStasenko 12/21/2011 12:39'!
translateX: px Y: py

	self subclassResponsibility! !
AthensBezierConverter initialize!
AthensPathBuilder initialize!
